module RainforestQA
  class Client
    include RainforestQA::API

    API_VERSION = '1'.freeze
    API_URL = 'https://app.rainforestqa.com/api'.freeze
    API_PREFIX = API_URL + "/" + API_VERSION + "/"

    attr_accessor :api_key

    def initialize(options = {})
      options.each do | k, v |
        if v.is_a?(Integer)
          raise RainforestQA::APIConfigurationError
        end
        send(:"#{k}=", v)
      end
      yield(self) if block_given?
    end

    def post(path, params = {})
      request(:post, path, params)
    end

    def get(path, params = {})
      request(:get, path, params)
    end

    def put(path, params = {})
      request(:put, path, params)
    end

    def delete(path, params = {})
      request(:delete, path, params)
    end

    private

    def connection(options = {})
      @connection ||= Faraday.new(API_URL, options) do | conn |
        conn.request   :encode_rfqa_json
        conn.adapter   :net_http
        conn.response  :parse_rfqa_json
        conn.response  :raise_error_rfqa_from_json
      end
    end

    def request(method, path, params = {})
      connection.send(method.to_sym) do | req |
        req.url API_PREFIX + path
        req.headers['CLIENT_TOKEN'] = api_key
        req.headers['Content-Type'] = 'application/json'
        req.body = params.to_json
      end
    rescue Faraday::Error::TimeoutError, Timeout::Error => error
      raise(RainforestQA::APITimeoutError.new(error))
    end
  end
end